<?php





































































































































$SQLF_FAIL_FUNC = false;



if ( !isset($SQLF_SAFE_VALUES)
 || !isset($SQLF_RECOVER)
 || !isset($SQLF_OVERRIDE))
{
 
 
 
 
 
 
 if ( !isset($argv)
 || $argc !== 2
 || $argv[1] !== 'sql_fragments.test_mode'
 || function_exists('sqlslashes'))
 {
 
 halt(7, "sql_fragments.php: globals not initialized");
 exit(1);
 }
}







if ( ! $SQLF_OVERRIDE) {











function cast_utf8_string($user_input) {
 return mb_convert_encoding((string)$user_input, 'UTF-8', 'UTF-8');
}




















function sqlf_value($given_val, $no_recover = false) {
 return sqlf_verify_or_recover('sqlv_value', $given_val, $no_recover);
}









function sqlv_value($given_val) {
 global $SQLF_SAFE_VALUES;

 
 
 
 $str_val = cast_utf8_string($given_val);

 
 
 
 
 
 
 
 if (false !== filter_var($str_val, FILTER_VALIDATE_INT, FILTER_FLAG_ALLOW_HEX)) {
 return $str_val;
 }

 
 
 $trimmed_val = trim($str_val, " \t\n\r");
 $no_leading_zeros = ltrim($trimmed_val, "0");

 
 if (false !== filter_var($no_leading_zeros, FILTER_VALIDATE_INT)) {
 
 
 
 return ' ' . $no_leading_zeros . ' ';
 }

 
 if (false !== filter_var($no_leading_zeros, FILTER_VALIDATE_FLOAT)) {
 return ' ' . $no_leading_zeros . ' ';
 }

 
 if ("" === $trimmed_val) {
 return "";
 }

 
 $upper_cased = mb_strtoupper($trimmed_val, 'UTF-8');
 $keyword_idx = array_search($upper_cased, $SQLF_SAFE_VALUES, 1);
 if (false !== $keyword_idx) {
 return ' ' . $SQLF_SAFE_VALUES[$keyword_idx] . ' ';
 }

 
 
 if (mb_strlen($trimmed_val) >= 2) {
 $first = mb_substr($trimmed_val, 0, 1, 'UTF-8');
 $last = mb_substr($trimmed_val, -1, 1, 'UTF-8');
 $inbetween = mb_substr($trimmed_val, 1, -1, 'UTF-8');
 if ($first === "'" && $first === $last) {
 return " '" . sqlf_escaped($inbetween) . "' ";
 }
 elseif ($first === '"' && $first === $last) {
 
 
 return ' "' . sqlf_escaped($inbetween) . '" ';
 }
 }

 return false;
}













































































function sqlf_escaped($given_val) {

 $str_val = cast_utf8_string($given_val);

 
 
 
 
 
 
 

 
 
 
 
 
 $unescape_tr = array(
 "\\0" => "\x00", 

 
 
 
 
 

 "\\b" => "\x08", 
 "\\n" => "\x0A", 
 "\\r" => "\x0D", 
 "\\t" => "\x09", 
 "\\Z" => "\x1A", 
 "\\\\" => "\\", 

 
 
 
 
 
 

 "\\%" => "\\%", 
 "\\_" => "\\_", 

 "\\" => "", 
 );

 $unescaped = strtr($str_val, $unescape_tr);

 
 
 
 $escaped = sqlslashes($unescaped);

 return $escaped;
}


















function sqlf_name($given_val, $no_recover = false) {
 return sqlf_verify_or_recover('sqlv_name', $given_val, $no_recover);
}





function sqlf_verify_or_recover($func, $given_val, $no_recover) {
 $verified = $func($given_val);
 if (false !== $verified) {
 return $verified;
 }
 return sqlf_try_recover($given_val, $no_recover);
}









function sqlv_name($given_val) {

 $str_val = cast_utf8_string($given_val);

 
 
 
 
 
 $sqlname_regex = '/^\s*[0-9a-zA-Z_\\.]*\*?\s*$/su';

 $options = array('options' => array('regexp' => $sqlname_regex));
 if (false !== filter_var($str_val, FILTER_VALIDATE_REGEXP, $options)) {
 return $str_val;
 }

 return false;
}





function sqlf_field_or_value($given_val, $no_recover = false) {
 return sqlf_any_atom($given_val, $no_recover);
}





function sqlf_field_with_keyword($given_val, $no_recover = false) {
 return sqlf_any_atom($given_val, $no_recover);
}














function sqlf_any($given_val) {
 return sqlf_map_list('sqlf_any_atom', $given_val, true);
}






function sqlf_any_atom($given_val, $no_recover = false) {

 $str_val = cast_utf8_string($given_val);

 
 
 
 $strip_regex = '/\b(?:ASC|DESC|(?:AS\s+)?[a-zA-Z0-9_]+)\s*$/sui';
 $modifiers = "";
 if (preg_match($strip_regex, $str_val, $matches)) {
 $modifiers = $matches[0];
 $str_val = mb_substr($str_val, 0, -mb_strlen($modifiers, 'UTF-8'), 'UTF-8');
 }

 $value = sqlv_value($str_val);
 if (false !== $value) {
 return $value . $modifiers;
 }

 $name = sqlv_name($str_val);
 if (false !== $name) {
 return $name . $modifiers;
 }

 return sqlf_try_recover($given_val, $no_recover);
}











function sqlf_escaped_eval_dq($given_val, $no_recover = false) {
 return sqlf_escaped($given_val, $no_recover);
}





function sqlf_tl_num_eval_dq($given_val, $no_recover = false) {
 return sqlf_tl_num($given_val, $no_recover);
}



















function sqlf_tl_num($given_val, $no_recover = false) {
 return sqlf_value($given_val, $no_recover);
}




function sqlf_limit_num ($gv, $nr = false) { return sqlf_value($gv, $nr); }
function sqlf_order_field ($gv, $nr = false) { return sqlf_any_atom($gv, $nr); }
function sqlf_order_way ($gv, $nr = false) { return sqlf_any_atom($gv, $nr); }
function sqlf_tl_num_list ($gv, $nr = false) { return sqlf_value_list($gv, $nr); }
function sqlf_tl_num_list_feature($gv, $nr = false) { return sqlf_value_list($gv, $nr);}
function sqlf_table ($gv, $nr = false) { return sqlf_name($gv, $nr); }
function sqlf_field ($gv, $nr = false) { return sqlf_name($gv, $nr); }
function sqlf_index ($gv, $nr = false) { return sqlf_name($gv, $nr); }
function sqlf_database ($gv, $nr = false) { return sqlf_name($gv, $nr); }







function sqlf_value_array($given_array, $no_recover = false) {
 return sqlf_map_array('sqlf_value', $given_array, $no_recover);
}
function sqlf_tl_num_array($given_array, $no_recover = false) {
 return sqlf_map_array('sqlf_tl_num', $given_array, $no_recover);
}
function sqlf_escaped_array($given_array, $no_recover = false) {
 return sqlf_map_array('sqlf_escaped', $given_array, $no_recover);
}
function sqlf_name_array($given_array, $no_recover = false) {
 return sqlf_map_array('sqlf_name', $given_array, $no_recover);
}
function sqlf_field_array($given_array, $no_recover = false) {
 return sqlf_map_array('sqlf_field', $given_array, $no_recover);
}








function sqlf_map_array($func, $given_array, $no_recover) {
 
 if (empty($given_array)) {
 return array();
 }
 return array_map($func, $given_array,
 array_fill(0, count($given_array), $no_recover));
}











function sqlf_value_list($given_list, $no_recover = false) {
 return sqlf_map_list('sqlf_value', $given_list, $no_recover);
}






function sqlf_field_with_keyword_list($given_list) {
 return sqlf_any($given_list);
}






function sqlf_map_list($func, $given_list, $no_recover) {
 $result_list = "";
 foreach (sqlf_split_list($given_list) as $list_item) {
 if ("," === $list_item) {
 
 $result_list .= ",";
 }
 else {
 
 $result_list .= $func($list_item, $no_recover);
 }
 }
 return $result_list;
}









function sqlf_split_list($given_val) {

 $str_val = cast_utf8_string($given_val);

 
 
 $regex = <<<HERE
/ ( #begin-capture...
   [^,\"\']*+ #...any leading non-commas...
   (?:   " (?:[^"\\\\]++|\\\\.|"")*+ \"   #...before doubly quoted strings...
       | ' (?:[^'\\\\]++|\\\\.|'')*+ \' ) #...or singly quoted strings...
   [^,]*+ #...and any trailing non-commas.
  ) #end-capture
| (,) #include commas in the result
/xsu
HERE;
 
 
 
 
 
 $parts = preg_split($regex, $str_val, -1,
 PREG_SPLIT_NO_EMPTY
 | PREG_SPLIT_DELIM_CAPTURE);
 return $parts;
}






function sqlf_try_recover($given_value, $no_recover) {
 global $SQLF_RECOVER;
 if (!$SQLF_RECOVER || $no_recover) {
 return sqlf_stop($given_value);
 }
 return sqlf_any($given_value);
}













function sqlf_stop($given_val) {
 global $SQLF_FAIL_FUNC;
 if ($SQLF_FAIL_FUNC) {
 return $SQLF_FAIL_FUNC($given_val);
 }

 halt(2,
 "The SQL value doesn't pass validation: "
 . $given_val
 . "\n"
 . print_r(debug_backtrace(false), true));
 exit(1);
}

} else {
function sqlf_any($given_value, $no_recover = false) { return $given_value; }
function sqlf_any_atom($given_value, $no_recover = false) { return $given_value; }
function sqlf_value($given_value, $no_recover = false) { return $given_value; }
function sqlf_value_list($given_value, $no_recover = false) { return $given_value; }
function sqlf_escaped($given_value, $no_recover = false) { return $given_value; }
function sqlf_name($given_value, $no_recover = false) { return $given_value; }
function sqlf_escaped_eval_dq($given_value, $no_recover = false) { return $given_value; }
function sqlf_tl_num_eval_dq($given_value, $no_recover = false) { return $given_value; }
function sqlf_tl_num_array($given_value, $no_recover = false) { return $given_value; }
function sqlf_value_array($given_value, $no_recover = false) { return $given_value; }
function sqlf_escaped_array($given_value, $no_recover = false) { return $given_value; }
function sqlf_field_array($given_value, $no_recover = false) { return $given_value; }
function sqlf_tl_num($given_value, $no_recover = false) { return $given_value; }
function sqlf_limit_num($given_value, $no_recover = false) { return $given_value; }
function sqlf_order_field($given_value, $no_recover = false) { return $given_value; }
function sqlf_order_way($given_value, $no_recover = false) { return $given_value; }
function sqlf_tl_num_list($given_value, $no_recover = false) { return $given_value; }
function sqlf_tl_num_list_feature($given_value, $no_recover = false) { return $given_value; }
function sqlf_field_or_value($given_value, $no_recover = false) { return $given_value; }
function sqlf_field_with_keyword($given_value, $no_recover = false) { return $given_value; }
function sqlf_field_with_keyword_list($given_value, $no_recover = false) { return $given_value; }
function sqlf_table($given_value, $no_recover = false) { return $given_value; }
function sqlf_field($given_value, $no_recover = false) { return $given_value; }
function sqlf_index($given_value, $no_recover = false) { return $given_value; }
function sqlf_database($given_value, $no_recover = false) { return $given_value; }
function sqlf_stop($given_value, $no_recover = false) { return $given_value; }
}











function sql_str($str) {
 return "'" . sqlslashes($str) . "'";
}











function sql_num($num) {
 return (string)(int)$num;
}







function sql_name($name) {
 return sqlf_name($name, true);
}














function sql_str_list($array) {
 return implode(',', array_map('sql_str', $array));
}

































































































































































































































































































































































































































































