<?




class XNLLoader extends BaseObject {

 var $server;
 var $connect;
 var $packages;
 var $qualified;
 var $functions;
 var $curl;
 var $config;
 var $parser;

 



 function XNLLoader( $config = false) {
 parent::BaseObject();
 $this->server = array();
 $this->connect = array();
 $this->packages = array();
 $this->functions = array();
 $this->qualified = array();
 $this->curl = new fetchCurl("POST");
 $this->config = array();
 $this->parser = new XMLParser();
 
 if ($config) {
 $this->addConfiguration($config);
 }
 }

 



 function addConfiguration($config) {
 if (!is_array($config)) return false;
 reset($config);
 while (list($key,$val) = each($config)) {
 if ($this->_isFile($val)) {
 $this->addConfigFile($val);
 } else {
 $this->addConfigURL($val);
 }
 }
 }

 function addConfigFile($filename) {
 if ($this->server["file"][$filename]) return false;
 $this->server["file"][$filename] = true;
 $xml = implode("",file($filename));
 $this->_loadConfig($xml);
 }

 function addConfigXML($xml) {
 $this->_loadConfig($xml);
 }

 function addConfigURL($url) {
 if ($this->server["url"][$url]) return false;
 $this->server["url"][$url] = true;
 
 $confurl = "$url?config=all";
 $this->curl->setURL($confurl);
 $xml = $this->curl->sendRequest();
 $defconnect = $this->_buildConnect($url);
 $this->_loadConfig($xml, "", $defconnect);
 }

 function addPackageInfo($package) {
 if (!isset($this->packages[$package])) return false;

 $url = $this->connect[$package]["uri"];
 
 $confurl = "$url?config=$package";
 $this->curl->setURL($confurl);
 $xml = $this->curl->sendRequest();
 if ($xml) $this->_loadConfig($xml, $package, $this->connect[$package]);
 }

 


 function resetConfig() {
 $this->packages = array();
 $this->server = array();
 $this->functions = array();
 $this->config = array();
 $this->connect = array();
 }

 


 function getConfigStruct() {
 $this->_rebuildConfig();
 return $this->config;
 }

 


 function getConfigXML() {
 $this->_rebuildConfig();
 $xml = new XMLParser();
 $xml->setStruct($this->config);
 return $xml->getXML();
 }

 function getConnect($package) {
 return $this->connect[$package];
 }

 function getPackages() {
 return $this->packages;
 }

 function getPackage($package) {
 if (!is_array($this->packages[$package])) return array();
 return $this->packages[$package];
 }

 function getFunctions($package) {
 if (!is_array($this->functions[$package])) return array();
 return $this->functions[$package];
 }


 function fetchPackages() {
 

 }

 function _isFile($url) {
 return !ereg("^http.?://",$url);
 }

 function _loadConfig($xml, $package = "", $defconnect = false) {
 global $DEBUG, $ICO_BUG;

 $this->parser->loadXML($xml);
 $struct = $this->parser->getStruct();

 $parser =& $this->parser->getXMLParser();
 if ($parser->getError() && GUIFactory::isInitialized()) {
 
 node_include("streaminfo");
 $info = new DebugStreamInfo();
 $info->addMessage(0, $ICO_BUG, "XMLLoader($package) Line ".$parser->getErrorLine().",".$parser->getErrorCol().": ".$parser->getErrorMessage(), $xml);
 }

 if (!$struct["CONFIGURATION"]) {
 
 BaseObject::addDebugMessage(i18n("Could not find a configuration section in XNL Configuration."), 
 "Package: $package\nXML: \n$xml");
 }

 $conf = $struct["CONFIGURATION"]["childs"];
 
 
 if ($conn = $conf["CONNECT"][0]) {
 $connect = $this->_loadConnect($conn,$defconnect);
 } else {
 $connect = $defconnect;
 }

 
 $packs = $conf["PACKAGES"][0]["childs"]["PACKAGE"];
 if (is_array($packs)) {
 reset($packs);
 while (list($key,$pack) = each($packs)) {
 $this->_loadPackage($pack, $package, $connect);
 }
 }

 return true;
 }

 function _loadPackage($pack, $mother, $defconnect = false) {
 $package = $pack["attributes"]["KEY"];
 if ($mother && $package) $package = "$mother.$package";
 else if ($mother) $package = $mother;
 
 if (!$package) {
 
 return false;
 }
 
 if ($conn = $pack["childs"]["CONNECT"][0]) {
 $connect = $this->_loadConnect($conn,$defconnect);
 } else if ($_mother = $this->_getMother($package)) {
 $connect = $this->getConnect($_mother);
 } else {
 $connect = $defconnect;
 }
 
 $this->packages[$package]["name"] = $pack["childs"]["NAME"][0]["value"];
 $this->packages[$package]["description"] = $pack["childs"]["DESCRIPTION"][0]["value"];

 $sig = $pack["childs"]["SIGNATURE"][0];

 $funcs = $pack["childs"]["FUNCTIONS"][0]["childs"]["FUNCTION"];
 if (is_array($funcs)) {
 while (list($key,$func) = each($funcs)) {
 $this->_loadFunction($func, $package, $sig);
 }
 }
 
 $this->connect[$package] = $connect;
 
 if ($package != $mother) $this->addPackageInfo($package);
 }

 function _loadFunction($func, $package, $signature) {
 
 $function = $func["attributes"]["KEY"];
 
 $funcel = array();

 $funcel["name"] = $func["childs"]["NAME"][0]["value"];
 $funcel["description"] = $func["childs"]["DESCRIPTION"][0]["value"];
 $funcel["file"] = $this->_loadFile($func["childs"]["FILE"][0], $function, $signature);
 $funcel["package"] = $package;
 $funcel["cache"] = $this->_loadCache($func["childs"]["CACHE"][0]);
 $funcel["params"] = array();
 
 $params = $func["childs"]["PARAMETERS"][0]["childs"]["PARAM"];
 if (is_array($params)) {
 reset($params);
 while (list($key,$param) = each($params)) {
 $pname = $param["attributes"]["KEY"];
 $paramel = array();
 $paramel["default"] = (is_array($param["childs"]["DEFAULT"]))?$param["childs"]["DEFAULT"][0]["value"]:false;
 if ($param["childs"]["REPLACE"][0]["value"] && strtolower($param["childs"]["REPLACE"][0]["attributes"]["TYPE"]) == "cdata") {
 $paramel["default"] = str_replace("< ![CDATA[","<![CDATA[", $paramel["default"]);
 $paramel["default"] = str_replace("]] >","]]>", $paramel["default"]);
 }
 $paramel["description"] = $param["childs"]["DESCRIPTION"][0]["value"];
 $optional = strtolower($param["childs"]["OPTIONAL"][0]["value"]);
 $paramel["optional"] = $this->_getBoolean($optional);
 $funcel["params"][$pname] = $paramel;
 }
 }
 
 $this->functions[$package][$function] = $funcel;
 }

 function _loadConnect($xmlconnect = array(), $defconnect = array()) {
 $rs = array();
 if ($uri = $xmlconnect["childs"]["URI"][0]["value"]) {
 $rs["uri"] = $uri;
 $rs["type"] = ($type = $xmlconnect["childs"]["METHOD"][0]["value"])?$type:(($type = $defconnect["type"])?$type:"");
 } else if ($uri = $defconnect["uri"]) {
 $rs["uri"] = $uri;
 $rs["type"] = ($type = $defconnect["type"])?$type:"";
 }
 return $rs;
 
 }

 function _loadFile($file, $function, $signature) {
 
 
 $file = array();
 if ($name = $file["childs"]["FIlENAME"][0]["value"]) {
 $file["filename"] = $name;
 } else {
 $file["filename"] = $function;
 }

 

 return $file;
 }

 function _loadCache($cache, $defCache = array()) {
 $rs = array();
 
 if (!isset($defCache["allowcache"])) $defCache["allowcache"] = true;
 if (!isset($defCache["interval"])) $defCache["interval"] = 0;
 if (!isset($defCache["republish"])) $defCache["republish"] = false;
 
 if (isset($cache["childs"]["ALLOWCACHE"][0]["value"])) {
 $val = $cache["childs"]["ALLOWCACHE"][0]["value"];
 $rs["allowcache"] = $this->_getBoolean($val);
 } else {
 $rs["allowcache"] = $defCache["allowcache"];
 }
 if (isset($cache["childs"]["INTERVAL"][0]["value"])) {
 $val = $cache["childs"]["INTERVAL"][0]["value"];
 $rs["interval"] = (int) $val * 60;
 } else {
 $rs["interval"] = $defCache["interval"];
 }
 if (isset($cache["childs"]["REPUBLISH"][0]["value"])) {
 $val = $cache["childs"]["REPUBLISH"][0]["value"];
 $rs["republish"] = $this->_getBoolean($val);
 } else {
 $rs["republish"] = $defCache["republish"];
 }
 
 return $rs;
 }

 function _getBoolean($val) {
 return (!$val || strtolower($val) == "false" || strtolower($val) == "no")?false:true;
 }

 function _getMother($package) {
 $parts = explode(".",$package);
 $mother = false;
 for ($i = count($parts) - 2; $i >= 0; $i--) {
 $pack = "";
 for ($j = 0; $j <= $i; $j++) {
 if ($pack) $pack .= ".";
 $pack .= $parts[$j];
 }
 if ($this->packages[$pack]) {
 $mother = $pack;
 break;
 }
 }
 return $mother;
 }

 function _buildConnect($url) {
 return array("uri"=>$url,"type"=>"POST");
 }

 function _rebuildConfig() {

 }
}

?>
